@@ -57,6 +57,7 @@
 #include "eog-save-dialog.h"
 #include "eog-horizontal-splitter.h"
 #include "eog-vertical-splitter.h"
+#include "eog-collection-view-iface.h"
 
 enum {
 	PROP_WINDOW_TITLE,
@@ -826,6 +827,76 @@ handle_right_click (EogWrapList *wlist, gint n, GdkEvent *event,
 #endif
 }
 
+/*
+ * This function is extracted from 
+ * File: nautilus/libnautilus-private/nautilus-file.c
+ * Revision: 1.309
+ * Author: Darin Adler <darin@bentspoon.com>
+ */
+static gboolean
+have_broken_filenames (void)
+{
+	static gboolean initialized = FALSE;
+	static gboolean broken;
+	
+	if (initialized) {
+		return broken;
+	}
+	
+	broken = g_getenv ("G_BROKEN_FILENAMES") != NULL;
+  
+	initialized = TRUE;
+  
+	return broken;
+}
+
+static char*
+get_title_from_uri (GnomeVFSURI *uri)
+{
+	gboolean validated = FALSE;
+	gboolean broken_filenames;
+	char *title = NULL;
+	char *utf8_title;
+
+	title = gnome_vfs_uri_extract_short_name (uri);
+	
+	if (title != NULL && gnome_vfs_uri_is_local (uri)) {
+		/* Support the G_BROKEN_FILENAMES feature of
+		 * glib by using g_filename_to_utf8 to convert
+		 * local filenames to UTF-8. Also do the same
+		 * thing with any local filename that does not
+		 * validate as good UTF-8.
+		 */
+		broken_filenames = have_broken_filenames ();
+		if (broken_filenames || !g_utf8_validate (title, -1, NULL)) {
+			utf8_title = g_locale_to_utf8 (title, -1, NULL, NULL, NULL);
+			if (utf8_title != NULL) {
+				g_free (title);
+				title = utf8_title;
+				/* Guaranteed to be correct utf8 here */
+				validated = TRUE;
+			}
+		} 
+		else if (!broken_filenames) {
+			/* name was valid, no need to re-validate */
+			validated = TRUE;
+		}
+	}
+	
+	if (!validated && !g_utf8_validate (title, -1, NULL)) {
+		if (title == NULL) {
+			title = g_strdup ("[Invalid Unicode]");
+		}
+		else {
+			utf8_title = eel_make_valid_utf8 (title);
+			g_free (title);
+			title = utf8_title;
+		}
+	}
+
+	return title;
+}
+
 static void
 eog_collection_view_get_prop (BonoboPropertyBag *bag,
 			      BonoboArg         *arg,
@@ -844,19 +915,23 @@ eog_collection_view_get_prop (BonoboPropertyBag *bag,
 	
 	switch (arg_id) {
 	case PROP_WINDOW_TITLE: {
-		gchar *base_uri;
-		gchar *title;
+		GnomeVFSURI *base_uri = NULL;
+		char *title = NULL;
 
-		/* FIXME: what should happen with this? */
-		base_uri = NULL; /*  eog_collection_model_get_base_uri (priv->model); */
+		base_uri = eog_image_list_get_base_uri (priv->model);
+		if (base_uri != NULL) {
+			title = get_title_from_uri (base_uri);
+		}
 
-		if (base_uri == NULL)
+		if (title == NULL) {
 			title = g_strdup (_("Collection View"));
-		else {
-			title = eel_format_uri_for_display (base_uri);
 		}
 
 		BONOBO_ARG_SET_STRING (arg, title);
+
+		if (base_uri != NULL) {
+			gnome_vfs_uri_unref (base_uri);
+		}
 		g_free (title);
 		break;
 	}
@@ -995,7 +1070,7 @@ model_size_changed (EogImageList *model, GList *id_list,  gpointer data)
 
 
 static void
-model_base_uri_changed (EogImageList *model, gpointer data)
+model_list_prepared (EogImageList *model, gpointer data)
 {
 	EogCollectionView *view;
 
@@ -1003,7 +1078,6 @@ model_base_uri_changed (EogImageList *model, gpointer data)
 	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
 
 	view = EOG_COLLECTION_VIEW (data);
-	g_print ("model_base_uri_changed ...\n");
 	update_title_text (view);	
 }
 
@@ -1109,22 +1183,22 @@ init_gconf_defaults (EogCollectionView *view)
 #endif
 }
 
-static gint
-load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
-	     CORBA_Environment *ev, void *closure)
+static EogImageList*
+get_collection_model (EogCollectionView *view)
 {
 	EogCollectionViewPrivate *priv;
-	EogCollectionView *view;
 
-	view = EOG_COLLECTION_VIEW (closure);
 	priv = view->priv;
 
-	if (text_uri == CORBA_OBJECT_NIL) return 0;
-
 	if (priv->model == NULL) {
 		priv->model = eog_image_list_new ();
 		eog_wrap_list_set_model (EOG_WRAP_LIST (priv->wraplist), priv->model);
 
+		g_signal_connect (G_OBJECT (priv->model), "list-prepared",
+				  G_CALLBACK (model_list_prepared),
+				  view);
+
+
 		/* construct widget */
 #if 0
 		g_signal_connect (G_OBJECT (priv->model), "image-added", 
@@ -1140,14 +1214,45 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 				  G_CALLBACK (model_base_uri_changed),
 				  list_view);
 #endif
-
 	}
 
-	eog_image_list_add_directory (priv->model, (gchar*)text_uri); 
+	return priv->model;
+}
+
+static gint
+load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
+	     CORBA_Environment *ev, void *closure)
+{
+	EogCollectionView *view;
+	EogImageList *model;
+	
+	view = EOG_COLLECTION_VIEW (closure);
+
+	if (text_uri == CORBA_OBJECT_NIL) return 0;
+
+	model = get_collection_model (view);
+
+	eog_image_list_add_directory (model, (gchar*)text_uri); 
 
 	return 0;
 }
 
+static void
+load_uri_list_cb (EogCollectionViewIface *collection, GList *uri_list, EogCollectionView *list_view)
+{
+
+	EogImageList *model;
+	
+	if (uri_list == NULL) return;
+
+	model = get_collection_model (list_view);
+
+	eog_image_list_add_files (model, uri_list);
+
+	g_list_foreach (uri_list, (GFunc) g_free, NULL);
+	g_list_free (uri_list);
+}
+
 static GtkWidget*
 create_user_interface (EogCollectionView *list_view)
 {
@@ -1234,7 +1339,6 @@ create_user_interface (EogCollectionView *list_view)
 	return vpaned;
 }
 
-
 static EogCollectionView *
 eog_collection_view_construct (EogCollectionView *list_view)
 {
@@ -1242,6 +1346,7 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	BonoboControl *control;
 	GtkWidget *root;
 	BonoboZoomable *zoomable;
+	EogCollectionViewIface *collection;
 
 	g_return_val_if_fail (list_view != NULL, NULL);
 	g_return_val_if_fail (EOG_IS_COLLECTION_VIEW (list_view), NULL);
@@ -1263,6 +1368,12 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	bonobo_object_add_interface (BONOBO_OBJECT (list_view),
 				     BONOBO_OBJECT (control));
 
+	/* interface GNOME::EOG::CollectionView */
+	collection = eog_collection_view_iface_new ();
+	g_signal_connect (collection, "load_uri_list", G_CALLBACK (load_uri_list_cb), list_view);
+	bonobo_object_add_interface (BONOBO_OBJECT (list_view),
+				     BONOBO_OBJECT (collection));
+
 
 	/* Interface Bonobo::Zoomable */
 	zoomable = bonobo_zoomable_new ();
